(*
 *               Newtons method to solve equation z^3 - 1 = 0
 *
 *
 *              code by Vojtech Hala, 11.12.1999, egg@atlas.cz
 *
 *)

{$G+,N+,E+ =generate 286 code and switch on math coprocessor}

type
 complex=record         { real and imaginary part of complex number }
   r,i:extended;
  end;

const
 iterations=30;         { number of iterations }
 epsilon=1e-16;

var
 x,y,color:longint;             { the pixel }
 cbank:word;                    { current video bank }
 z:complex;                     { complex number to calculate with }


procedure f(z1:complex;var z2:complex);
{ f(z) = (2z^3 + 1) / 3z^2 }
var
 zz1,zz2:complex;
 a,b,c,d,e,f,val1,ang1,val2,ang2:extended;
begin
z2.r:=0;
z2.i:=0;
if (z1.r=0)and(z1.i=0)then exit;
a:=z1.r; b:=z1.i;
c:=2*( a*a*a-3*a*b*b)+1;
d:=2*(-b*b*b+3*b*a*a);
e:=3*(a*a-b*b); f:=6*a*b;
z2.r:=(c*e+f*d)/(e*e+f*f);
z2.i:=(d*e-c*f)/(e*e+f*f);
end;

function Distance(z1,z2:complex):extended;
{ returns distance of two complex numbers in Gauss plane }
begin
Distance:=sqrt((z1.r-z2.r)*(z1.r-z2.r)+(z1.i-z2.i)*(z1.i-z2.i));
end;

function Julia(z:complex):longint;
{ used to draw the Julia set fractal }
var
 n:longint;
 r1,r2,r3:complex; { atractors of the dynamic system }
 h:byte;           { stores which atractor does it go to }
begin
r1.r:=1;r1.i:=0;
r2.r:=-0.5;r2.i:=sqrt(3)/2;
r3.r:=-0.5;r3.i:=-sqrt(3)/2;
h:=0;
for n:=1 to iterations do begin
  f(z,z); { iterating the complex function }
  if (Distance(z,r1)<epsilon) then begin
    h:=1;
    break; { break if it's close to the solution }
   end;
  if (Distance(z,r2)<epsilon) then begin
    h:=2;
    break;
   end;
  if (Distance(z,r3)<epsilon) then begin
    h:=3;
    break;
   end;
 end;
if h>0 then julia:=63*n div iterations+64*h else julia:=0;
{ returns pixel color }
end;{Julia}

procedure PutPixel(x,y:word;color:byte);assembler;
{ in VESA mode 101h (640x480x256) }
asm
        mov  ax,y
        mov  dx,640
        mul  dx
        add  ax,x
        adc  dx,0
        mov  di,ax              { compute offset }
        cmp  dx,cbank
        je   @BankOK
        mov  ax,4f05h
        mov  bx,0
        int  10h                { set video bank }
        mov  cBank,dx
@BankOK:mov  es,[SegA000]
        mov  al,color
        mov  [es:di],al         { write to video memory }
end;{PutPixel}

procedure SetRGB(color,r,g,b:byte);assembler;
{ define RGB values for specified color index }
asm
 mov dx,03c8h
 mov al,color
 out dx,al
 inc dx
 mov al,r
 out dx,al
 mov al,g
 out dx,al
 mov al,b
 out dx,al
end;{SetRGB}

procedure SetTextMode;assembler;
{ returns standard text mode 80x25 }
asm
mov ax,3
int 10h
end;{SetTextMode}

procedure SetSvga(mode:word);assembler;
{ kicks you into the specified VESA mode }
asm
mov ax,4f02h
mov bx,mode
int 10h
end;{SetSvga}


procedure SetPalette1;
{ this prepares R/G/B palette }
var i:byte;
begin
for i:=  0 to  63 do SetRGB(i,0,0,0);
for i:= 64 to 127 do SetRGB(i,63-i,0,0);
for i:=128 to 191 do SetRGB(i,0,63-i-128,0);
for i:=192 to 255 do SetRGB(i,0,0,63-i-192);
{ this one is good to see the mathematical principle }
end;{SetPalette1}

procedure SetPalette2;
{ but this one is a little bit more aesthetic }
var i:byte;
begin
for i:=0 to 63 do SetRGB(i,0,0,i);
for i:=64 to 127 do SetRGB(i,i-64,i-64,127-i);
for i:=128 to 191 do SetRGB(i,63,127-i,0);
for i:=0 to 63 do SetRGB(i+192,63-i,0,0);
end;{SetPalette2}


function ReadKey:char;assembler;
{ almost equal to crt.readkey }
asm
mov ax,0
int 16h
end;

{ --- The program body ---}

begin
SetSvga($101);       { set mode 640x480x256 }
SetPalette1;         { set 256 colors RGB table }
for y:=0 to 479 do { complex function is called for each pixel }
 for x:=0 to 639 do begin
   z.r:=(x-320)/180;{ compute the complex number }
   z.i:=(240-y)/180;{ in the middle of the pixel }
   color:=Julia(z);
   PutPixel(x,y,color);
  end;
for x:=30 to 50 do for y:=355 downto 100 do PutPixel(640-x,456-y,355-y);
{ display used palette entry }
readkey;
SetPalette2; { a little change }
readkey;
SetTextMode; { return to system }
end.